/*
 *  Copyright (C) 2003 Burton M. Strauss III <Burton@ntopsupport.com>
 *
 *  		       http://www.ntop.org/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* ****************************************************************** */

#include "ntop.h"
#include "globals-report.h"

static void setPluginStatus(char * status);

/* ****************************** */

static int initParmsfunct(void) {
  traceEvent(CONST_TRACE_NOISY, "PARMS: Silly user, it makes no sense to 'activate' this one...");
  setPluginStatus("Activating this plugin is not necessary.");
  return(-1);
}

/* ****************************** */

static void handleparmsHTTPrequest(char* url) {
  char buf[1024], buf1[32], buf2[32];
  float percentage, err;
  struct in_addr theDest;
  int i;

  sendHTTPHeader(FLAG_HTTP_TYPE_HTML, 0);
  printHTMLheader("Reconfigurable parameters", 0);

  sendString("<CENTER>\n<HR>\n");

  if(url != NULL) {
    char *key, *value;

    key = strtok(url, "=");
    if(key != NULL) value = strtok(NULL, "="); else value = NULL;

    if(value && key) {
      if(strcmp(key, "packetdecoding") == 0) {
        myGlobals.enablePacketDecoding =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: PacketDecoding set to %d", myGlobals.enablePacketDecoding);
      } else if(strcmp(key, "stickyhosts") == 0) {
        myGlobals.stickyHosts =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: StickyHosts set to %d", myGlobals.stickyHosts);
      } else if(strcmp(key, "tracklocalonly") == 0) {
        myGlobals.trackOnlyLocalHosts =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: TrackLocalHostsOnly set to %d", myGlobals.trackOnlyLocalHosts);
      } else if(strcmp(key, "refreshrate") == 0) {
        if(atoi(value) <= 0)
          myGlobals.refreshRate = DEFAULT_NTOP_AUTOREFRESH_INTERVAL;
        else if(atoi(value) < PARM_MIN_WEBPAGE_AUTOREFRESH_TIME)
          myGlobals.refreshRate = PARM_MIN_WEBPAGE_AUTOREFRESH_TIME;
        else
          myGlobals.refreshRate = atoi(value);
        traceEvent(CONST_TRACE_INFO, "PARMS: RefreshRate set to %d", myGlobals.refreshRate);
      } else if(strcmp(key, "nomac") == 0) {
        myGlobals.dontTrustMACaddr =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: NoMac set to %d", myGlobals.dontTrustMACaddr);
      } else if(strcmp(key, "linesperpage") == 0) {
        if(atoi(value) <= 0)
          myGlobals.maxNumLines = CONST_NUM_TABLE_ROWS_PER_PAGE;
        else
          myGlobals.maxNumLines = atoi(value);
        traceEvent(CONST_TRACE_INFO, "PARMS: LinesPerPage set to %d", myGlobals.maxNumLines);
      } else if(strcmp(key, "tracelevel") == 0) {
        myGlobals.traceLevel =
            max(min(atoi(value), CONST_NOISY_TRACE_LEVEL), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: TraceLevel set to %d", myGlobals.traceLevel);
      } else if(strcmp(key, "disablesessions") == 0) {
        myGlobals.enableSessionHandling =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: SessionHandling set to %d", myGlobals.enableSessionHandling);
      } else if(strcmp(key, "debug") == 0) {
        myGlobals.debugMode =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: SessionHandling set to %d", myGlobals.debugMode);
      } else if(strcmp(key, "logextra") == 0) {
        myGlobals.logExtra =
            max(min(atoi(value), 2), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: LogExtra set to %d", myGlobals.logExtra);
      } else if(strcmp(key, "instantsessions") == 0) {
        myGlobals.disableInstantSessionPurge =
            max(min(atoi(value), 1), 0);
        traceEvent(CONST_TRACE_INFO, "PARMS: InstantSessionPurge set to %d", myGlobals.disableInstantSessionPurge);
      }
    }
  }

  /* *************************************** */

  sendString("<p>This plugin is used to change ntop's command line parameter settings.\n"
                "It only presents those parameters which may be changed during an ntop run.</p>\n"
             "<p><b>Changes take affect immediately.</b></p>\n"
             "<br>\n");

  sendString("<table border=0>\n<tr><td><table border>");

  sendString("<tr "TR_ON">\n"
               "<th "TH_BG" align=\"left\" width=\"60\">Parameter</th>\n"
               "<th "TH_BG" align=\"left\" width=\"175\">Command Line</th>\n"
               "<th "TH_BG" align=\"left\" width=\"240\">Setting value / choices</th>\n"
               "<th "TH_BG" align=\"left\">Notes (mostly cribs from <i>man ntop</i>)</th>\n"
               "<th "TH_BG" align=\"left\">&nbsp;</th>\n</tr>\n");

/*TODO***********************/
//  enableSuspiciousPacketDump -q | --create-suspicious-packets
// Complex - stuff in initialize.c and ntop.c must be replicated, but is it worth while?
/*TODO***********************/

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Packet Decoding</th>\n"
             "<td "TD_BG">-b | --disable-decoders</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"packetdecoding\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.enablePacketDecoding == 1 ? " checked" : "",
              "Yes") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"packetdecoding\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.enablePacketDecoding == 0 ? " checked" : "",
              "No") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">This flag disables protocol decoders (e.g. DNS, NetBIOS). Use it for "
              "better performance or if you feel ntop has problem handling some protocols</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Sticky Hosts</th>\n"
             "<td "TD_BG">-c | --sticky-hosts</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"stickyhosts\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.stickyHosts == 1 ? " checked" : "",
              "Yes") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"stickyhosts\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.stickyHosts == 0 ? " checked" : "",
              "No") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">By default idle hosts are periodically purged from memory. "
              "Use this flag to prevent idle hosts from being purged from memory.<br>"
              "<i>NOTE: if idle hosts are kept in memory you will most likely "
              "observer ntop using significantly more memory.</i></td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Lines per page</th>\n"
             "<td "TD_BG">-e | --max-table-rows</td>\n"
             "<td "TD_BG" align=\"left\">\n"
             "<form ACTION=/plugins/parms METHOD=GET>\n"
             "<input NAME=\"linesperpage\" SIZE=\"5\" VALUE=\"");

  if(snprintf(buf, sizeof(buf), "%d", myGlobals.maxNumLines) < 0)
    BufferTooShort();
  sendString(buf);

  sendString("\">&nbsp;lines\n</td>\n");
  sendString("<td "TD_BG">Is the maximum number of HTML table rows that ntop will display "
             "on a single page.</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Track Local Hosts</th>\n"
             "<td "TD_BG">-g | --track-local-hosts</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracklocalonly\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.trackOnlyLocalHosts == 1 ? " checked" : "",
              "Only") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracklocalonly\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.trackOnlyLocalHosts == 0 ? " checked" : "",
              "No") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">Use this flag to tell ntop that you care only about local "
             "hosts. <i>(you must have used -m to specify them at startup time)</i> "
             "This flag is useful on large networks or those that see many hosts, "
             "e.g. a border router or gateway, if you don't care about the others.</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">MAC Addresses</th>\n"
             "<td "TD_BG">-o | --no-mac</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"nomac\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.dontTrustMACaddr == 1 ? " checked" : "",
              "Yes") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"nomac\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.dontTrustMACaddr == 0 ? " checked" : "",
              "No") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">Specifies that ntop should not trust MAC addresses but "
             "just the IP addresses. This option is useful whenever ntop is running "
             "on an interface where MAC addresses cannot be really trusted (e.g. "
             "port/VLAN mirror)</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Refresh Time</th>\n"
             "<td "TD_BG">-r | --refresh-time</td>\n"
             "<td "TD_BG" align=\"left\">\n"
             "<form ACTION=/plugins/parms METHOD=GET>\n"
             "<input NAME=\"refreshrate\" SIZE=\"5\" VALUE=\"");

  if(snprintf(buf, sizeof(buf), "%d", myGlobals.refreshRate) < 0)
    BufferTooShort();
  sendString(buf);

  sendString("\">&nbsp;seconds\n</td>\n");
  sendString("<td "TD_BG">Specifies the delay (in seconds) between screen updates "
             "(the default is 3 seconds). <i>Please note that if the delay is very "
             "short, ntop might not be able to process all the network traffic.</i></td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
               "<th "TH_BG" align=\"left\">Trace Level</th>\n"
               "<td "TD_BG">-t | --trace-level</td>\n"
               "<td "TD_BG" align=\"left\"><form ACTION=/plugins/parms METHOD=GET>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracelevel\" VALUE=%d%s>%s (%d)<br>\n",
              CONST_ERROR_TRACE_LEVEL,
              myGlobals.traceLevel == CONST_FATALERROR_TRACE_LEVEL ? " checked" : "",
              "Fatal Errors only",
              CONST_ERROR_TRACE_LEVEL) < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracelevel\" VALUE=%d%s>%s (%d)<br>\n",
              CONST_ERROR_TRACE_LEVEL,
              myGlobals.traceLevel == CONST_ERROR_TRACE_LEVEL ? " checked" : "",
              "Errors only",
              CONST_ERROR_TRACE_LEVEL) < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracelevel\" VALUE=%d%s>%s (%d)<br>\n",
              CONST_WARNING_TRACE_LEVEL,
              myGlobals.traceLevel == CONST_WARNING_TRACE_LEVEL ? " checked" : "",
              "Warnings and Errors",
              CONST_WARNING_TRACE_LEVEL) < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracelevel\" VALUE=%d%s>%s (%d)<br>\n",
              CONST_INFO_TRACE_LEVEL,
              myGlobals.traceLevel == CONST_INFO_TRACE_LEVEL ? " checked" : "",
              "Informational and more serious",
              CONST_INFO_TRACE_LEVEL) < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"tracelevel\" VALUE=%d%s>%s (%d)<br>\n",
              CONST_NOISY_TRACE_LEVEL,
              myGlobals.traceLevel == CONST_NOISY_TRACE_LEVEL ? " checked" : "",
              "All ('noisy')",
              CONST_NOISY_TRACE_LEVEL) < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
 sendString("<td "TD_BG">This flag specifies the level of ntop tracings on stdout "
            "or to the system log. <i>Unlike the command line option, setting "
            "--trace-level 5 (called 'noisy' here) does NOT set --log-extra 1.</i></td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Session Handling</th>\n"
             "<td "TD_BG">-z | --disable-sessions</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"disablesessions\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.enableSessionHandling == 1 ? " checked" : "",
              "Enabled") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"disablesessions\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.enableSessionHandling == 0 ? " checked" : "",
              "Disabled") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">This  flag  disables  TCP session tracking.  Use it for better "
              "performance or when you don't really need/care to track sessions.</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

#ifndef WIN32

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Debug</th>\n"
             "<td "TD_BG">-K | --enable-debug</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"debug\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.debugMode == 1 ? " checked" : "",
              "Yes") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"debug\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.debugMode == 0 ? " checked" : "",
              "No") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">This flag enables 'debug mode'. Debug mode does the following:<br>\n"
              "<ul><li>Prevents the fork() call for a read-only web page. <i>On systems such "
              "as Linux, which implement copy-on-write fork(), this is a very efficient method "
              "of obtaining a complete, read-only duplicate of ntop's internal tables for "
              "reporting.  On systems which do not implement copy-on-write fork(), this can be "
              "a very expensive process.</i></li>\n"
              "<li>Activates the automated backtrace trap during startup.<b>This is NOT set "
              "if you use this plugin to change the Debug flag.</b> <i>However, no worries, "
              "as the code - in initialize.c - is presently #if 0, so it's not executed</i></li>\n"
              "<li>Adds a mutex status report to the configuration pages (info.html and "
              "textinfo.html)</li>\n"
              "</ul>\n</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");
#endif /* WIN32 */

  /* ********** */

  sendString("<tr "TR_ON">\n"
               "<th "TH_BG" align=\"left\">Log Extra</th>\n"
               "<td "TD_BG">--log-extra</td>\n"
               "<td "TD_BG" align=\"left\"><form ACTION=/plugins/parms METHOD=GET>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"logextra\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.logExtra == 0 ? " checked" : "",
              "None") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"logextra\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.logExtra == 1 ? " checked" : "",
              "file:line") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"logextra\" VALUE=%d%s>%s<br>\n",
              2,
              myGlobals.logExtra == 2 ? " checked" : "",
              "MSGID") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">Add extra information to log messages.</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");

  /* ********** */

  sendString("<tr "TR_ON">\n"
             "<th "TH_BG" align=\"left\">Instant Session Purge</th>\n"
             "<td "TD_BG">--disable-instantsessionpurge</td>\n"
             "<td "TD_BG" align=\"left\"><form action=\"/plugins/parms\" method=get>");

  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"instantsessions\" VALUE=%d%s>%s<br>\n",
              1,
              myGlobals.disableInstantSessionPurge == 1 ? " checked" : "",
              "Enabled") < 0)
    BufferTooShort();
  sendString(buf);
  if(snprintf(buf,
              sizeof(buf),
              "<input TYPE=\"radio\" NAME=\"instantsessions\" VALUE=%d%s>%s<br>\n",
              0,
              myGlobals.disableInstantSessionPurge == 0 ? " checked" : "",
              "Disabled") < 0)
    BufferTooShort();
  sendString(buf);
  sendString("</td>\n");
  sendString("<td "TD_BG">ntop sets completed sessions as 'timed out' and then "
             "purges them almost instantly, which is not the behavior you might "
             "expect from the purge timeouts in globals-defines.h.\n"
             "This switch makes ntop respect the timeouts for completed sessions."
             "<i>It is NOT the default because a busy web server may have 100s or "
             "1000s of completed sessions and this would <b>significantly</b> "
             "increase the amount of memory ntop uses.</td>\n");
  sendString("<td align=\"center\">\n<input TYPE=submit VALUE=Set></form>\n</td>\n</tr>\n");


  /* ********** */

  sendString("</table>\n</tr>\n");

  sendString("<tr><td>"
	     "<b>NOTES</b>:<ol>"
	     "<li>You are not able to make this plugin 'active' as that makes no sense.</li>"
             "<li>Many of the command line parameters are one-way switches or are used only "
                 "during ntop's initialization.  Those parameters can not be changed during "
                 "the run and are not shown here (Use About | Configuration to see their "
                 "values)</li>"
             "<li>While the parameters may take affect instantly, it can take considerable "
                 "time for the effects of some changes to be visible.  For example, track "
                 "only local hosts will not remove the remote host records. So they will "
                 "still appear in ntop's reports until they naturally age out of the tables "
                 "as 'idle' hosts.</li>"
	     "</ol></td></tr>\n");
  sendString("</table></center><p>\n");

  /* *************************************** */

    
  sendString("<p align=right>[ Back to <a href=\"../" STR_SHOW_PLUGINS "\">plugins</a> ]&nbsp;</p>\n");
  printHTMLtrailer();
}

/* ****************************** */

static PluginInfo parmsPluginInfo[] = {
  { "parmsPlugin",
    "This plugin is used modify - on the fly - a subset of ntop's command line parameters",
    "2.3", /* version */
    "<A HREF=\"http://www.ntopsupport.com\">Burton Strauss</A>",
    "parms", /* http://<host>:<port>/plugins/parms */
    0, /* Inactive (active makes no sense for this) */
    1, /* Inactive setup */
    initParmsfunct,    /* InitFunc   */
    NULL,    /* TermFunc   */
    NULL,    /* PluginFunc */
    handleparmsHTTPrequest,
    NULL,    /* no capture */
    NULL     /* no status */
  }
};

/* ***************************************** */

/* Plugin entry fctn */
#ifdef MAKE_STATIC_PLUGIN
PluginInfo* parmsPluginEntryFctn(void)
#else
     PluginInfo* PluginEntryFctn(void)
#endif
{
  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "PARMS: Welcome to %s. (C) 2003 by Burton Strauss",
	     parmsPluginInfo->pluginName);

  return(parmsPluginInfo);
}

/* This must be here so it can access the struct PluginInfo, above */
static void setPluginStatus(char * status)
   {
       if (parmsPluginInfo->pluginStatusMessage != NULL)
           free(parmsPluginInfo->pluginStatusMessage);
       if (status == NULL) {
           parmsPluginInfo->pluginStatusMessage = NULL;
       } else {
           parmsPluginInfo->pluginStatusMessage = strdup(status);
       }
   }

